import { __assign } from "tslib";
/* eslint-disable react-hooks/exhaustive-deps -- will come back and fix separately */
import * as React from 'react';
import { IconButton } from '../../compat/Button';
import { Label } from '../../Label';
import { Icon } from '../../Icon';
import { getId, KeyCodes, calculatePrecision, classNamesFunction, precisionRound, getNativeProps, divProperties, } from '../../Utilities';
import { getArrowButtonStyles } from './SpinButton.styles';
import { KeyboardSpinDirection } from './SpinButton.types';
import { Position } from '../../Positioning';
import { useBoolean, useSetTimeout, useControllableValue, useWarnings } from '@fluentui/react-hooks';
var getClassNames = classNamesFunction();
var COMPONENT_NAME = 'SpinButton';
var useComponentRef = function (props, input, value) {
    React.useImperativeHandle(props.componentRef, function () { return ({
        get value() {
            return value;
        },
        focus: function () {
            if (input.current) {
                input.current.focus();
            }
        },
    }); }, [value]);
};
export var SpinButtonBase = React.forwardRef(function (props, ref) {
    var _a = props, _b = _a.disabled, disabled = _b === void 0 ? false : _b, _c = _a.label, label = _c === void 0 ? '' : _c, _d = _a.min, min = _d === void 0 ? 0 : _d, _e = _a.max, max = _e === void 0 ? 100 : _e, _f = _a.step, step = _f === void 0 ? 1 : _f, _g = _a.labelPosition, labelPosition = _g === void 0 ? Position.start : _g, iconProps = _a.iconProps, _h = _a.incrementButtonIcon, incrementButtonIcon = _h === void 0 ? { iconName: 'ChevronUpSmall' } : _h, incrementButtonAriaLabel = _a.incrementButtonAriaLabel, _j = _a.decrementButtonIcon, decrementButtonIcon = _j === void 0 ? { iconName: 'ChevronDownSmall' } : _j, decrementButtonAriaLabel = _a.decrementButtonAriaLabel, ariaLabel = _a.ariaLabel, ariaDescribedBy = _a.ariaDescribedBy, customUpArrowButtonStyles = _a.upArrowButtonStyles, customDownArrowButtonStyles = _a.downArrowButtonStyles, theme = _a.theme, ariaPositionInSet = _a.ariaPositionInSet, ariaSetSize = _a.ariaSetSize, ariaValueNow = _a.ariaValueNow, ariaValueText = _a.ariaValueText, className = _a.className, inputProps = _a.inputProps, iconButtonProps = _a.iconButtonProps, styles = _a.styles;
    var input = React.useRef(null);
    var _k = useBoolean(false), isFocused = _k[0], _l = _k[1], setTrueIsFocused = _l.setTrue, setFalseIsFocused = _l.setFalse;
    var _m = React.useState(KeyboardSpinDirection.notSpinning), keyboardSpinDirection = _m[0], setKeyboardSpinDirection = _m[1];
    var _o = useSetTimeout(), setTimeout = _o.setTimeout, clearTimeout = _o.clearTimeout;
    var _p = useControllableValue(props.value, props.defaultValue !== undefined ? props.defaultValue : String(min || '0')), value = _p[0], setValue = _p[1];
    if (process.env.NODE_ENV !== 'production') {
        // eslint-disable-next-line react-hooks/rules-of-hooks -- build-time conditional
        useWarnings({
            name: COMPONENT_NAME,
            props: props,
            mutuallyExclusive: { value: 'defaultValue' },
        });
    }
    useComponentRef(props, input, value);
    var callCalculatePrecision = function (calculatePrecisionProps) {
        var _a = calculatePrecisionProps.precision, precision = _a === void 0 ? Math.max(calculatePrecision(calculatePrecisionProps.step), 0) : _a;
        return precision;
    };
    var _q = props.value, initialValue = _q === void 0 ? props.defaultValue : _q;
    if (initialValue === undefined) {
        initialValue = typeof min === 'number' ? String(min) : '0';
    }
    var internalState = React.useRef({
        inputId: getId('input'),
        labelId: getId('Label'),
        lastValidValue: initialValue,
        spinningByMouse: false,
        valueToValidate: undefined,
        precision: callCalculatePrecision(props),
        currentStepFunctionHandle: -1,
        initialStepDelay: 400,
        stepDelay: 75,
    }).current;
    if (props.value !== undefined) {
        internalState.lastValidValue = props.value;
    }
    internalState.precision = callCalculatePrecision(props);
    var classNames = getClassNames(styles, {
        theme: theme,
        disabled: disabled,
        isFocused: isFocused,
        keyboardSpinDirection: keyboardSpinDirection,
        labelPosition: labelPosition,
        className: className,
    });
    var nativeProps = getNativeProps(props, divProperties, [
        'onBlur',
        'onFocus',
        'className',
    ]);
    // Validate function to use if one is not passed in
    var defaultOnValidate = function (valueProp) {
        if (valueProp === null || valueProp.trim().length === 0 || isNaN(Number(valueProp))) {
            return internalState.lastValidValue;
        }
        var newValue = Math.min(max, Math.max(min, Number(valueProp)));
        return String(newValue);
    };
    // Increment function to use if one is not passed in
    var defaultOnIncrement = function (valueProp) {
        var newValue = Math.min(Number(valueProp) + Number(step), max);
        newValue = precisionRound(newValue, internalState.precision);
        return String(newValue);
    };
    // Increment function to use if one is not passed in
    var defaultOnDecrement = function (valueProp) {
        var newValue = Math.max(Number(valueProp) - Number(step), min);
        newValue = precisionRound(newValue, internalState.precision);
        return String(newValue);
    };
    var validate = function (ev) {
        if (value !== undefined &&
            internalState.valueToValidate !== undefined &&
            internalState.valueToValidate !== internalState.lastValidValue) {
            var newValue = onValidate(internalState.valueToValidate, ev);
            // Done validating this value, so clear it
            internalState.valueToValidate = undefined;
            if (newValue !== undefined) {
                internalState.lastValidValue = newValue;
                setValue(newValue);
            }
            else {
                // Value was invalid. Reset state to last valid value.
                setValue(internalState.lastValidValue);
            }
        }
    };
    var onIncrement = function (valueProp) {
        if (props.onIncrement) {
            return props.onIncrement(valueProp);
        }
        else {
            return defaultOnIncrement(valueProp);
        }
    };
    var onDecrement = function (valueProp) {
        if (props.onDecrement) {
            return props.onDecrement(valueProp);
        }
        else {
            return defaultOnDecrement(valueProp);
        }
    };
    var onValidate = function (valueProp, ev) {
        if (props.onValidate) {
            return props.onValidate(valueProp, ev);
        }
        else {
            return defaultOnValidate(valueProp);
        }
    };
    var onChange = React.useCallback(function () {
        /**
         * A noop input change handler. Using onInput instead of onChange was meant to address an issue
         * which apparently has been resolved in React 16 (https://github.com/facebook/react/issues/7027).
         * The no-op onChange handler was still needed because React gives console errors if an input
         * doesn't have onChange.
         *
         * TODO (Fabric 8?) - switch to just calling onChange (this is a breaking change for any tests,
         * ours or 3rd-party, which simulate entering text in a SpinButton)
         */
    }, []);
    //  The method is needed to ensure we are updating the actual input value.
    //  without this our value will never change (and validation will not have the correct number)
    var onInputChange = React.useCallback(function (event) {
        var element = event.target;
        var elementValue = element.value;
        internalState.valueToValidate = elementValue;
        setValue(elementValue);
    }, []);
    var onFocus = React.useCallback(function (ev) {
        // We can't set focus on a non-existing element
        if (!input.current) {
            return;
        }
        if (internalState.spinningByMouse || keyboardSpinDirection !== KeyboardSpinDirection.notSpinning) {
            stop();
        }
        input.current.select();
        setTrueIsFocused();
        if (props.onFocus) {
            props.onFocus(ev);
        }
    }, [props.onFocus]);
    var onBlur = React.useCallback(function (ev) {
        validate(ev);
        setFalseIsFocused();
        if (props.onBlur) {
            props.onBlur(ev);
        }
    }, [props.onBlur]);
    /**
     * Update the value with the given stepFunction
     * @param shouldSpin - should we fire off another updateValue when we are done here? This should be true
     * when spinning in response to a mouseDown
     * @param stepFunction - function to use to step by
     * @param event - The event that triggered the updateValue
     */
    var updateValue = function (shouldSpin, stepDelay, stepFunction, event) {
        var newValue = stepFunction(value || '', event);
        if (newValue !== undefined) {
            setValue(newValue);
        }
        if (internalState.spinningByMouse !== shouldSpin) {
            internalState.spinningByMouse = shouldSpin;
        }
        if (shouldSpin) {
            internalState.currentStepFunctionHandle = setTimeout(function () {
                updateValue(shouldSpin, internalState.stepDelay, stepFunction, event);
            }, stepDelay);
        }
    };
    // Stop spinning (clear any currently pending update and set spinning to false)
    var stop = React.useCallback(function () {
        if (internalState.currentStepFunctionHandle >= 0) {
            clearTimeout(internalState.currentStepFunctionHandle);
            internalState.currentStepFunctionHandle = -1;
        }
        if (internalState.spinningByMouse || keyboardSpinDirection !== KeyboardSpinDirection.notSpinning) {
            internalState.spinningByMouse = false;
            setKeyboardSpinDirection(KeyboardSpinDirection.notSpinning);
        }
    }, [internalState.currentStepFunctionHandle, keyboardSpinDirection, KeyboardSpinDirection.notSpinning]);
    //  Handle keydown on the text field. We need to update
    //  the value when up or down arrow are depressed
    var handleKeyDown = React.useCallback(function (event) {
        // eat the up and down arrow keys to keep focus in the spinButton
        // (especially when a spinButton is inside of a FocusZone)
        if (event.which === KeyCodes.up || event.which === KeyCodes.down || event.which === KeyCodes.enter) {
            event.preventDefault();
            event.stopPropagation();
        }
        if (props.disabled) {
            stop();
            return;
        }
        var spinDirection = KeyboardSpinDirection.notSpinning;
        switch (event.which) {
            case KeyCodes.up:
                spinDirection = KeyboardSpinDirection.up;
                updateValue(false /* shouldSpin */, internalState.initialStepDelay, onIncrement, event);
                break;
            case KeyCodes.down:
                spinDirection = KeyboardSpinDirection.down;
                updateValue(false /* shouldSpin */, internalState.initialStepDelay, onDecrement, event);
                break;
            case KeyCodes.enter:
                validate(event);
                break;
            case KeyCodes.escape:
                if (value !== internalState.lastValidValue) {
                    setValue(internalState.lastValidValue);
                }
                break;
            default:
                break;
        }
        // style the increment/decrement button to look active
        // when the corresponding up/down arrow keys trigger a step
        if (keyboardSpinDirection !== spinDirection) {
            setKeyboardSpinDirection(spinDirection);
        }
    }, [KeyCodes]);
    // Make sure that we have stopped spinning on keyUp
    // if the up or down arrow fired this event
    var handleKeyUp = React.useCallback(function (ev) {
        if (props.disabled || ev.which === KeyCodes.up || ev.which === KeyCodes.down) {
            stop();
            return;
        }
    }, [props.disabled]);
    var onIncrementMouseDown = React.useCallback(function (event) {
        updateValue(true /* shouldSpin */, internalState.initialStepDelay, onIncrement, event);
    }, []);
    var onDecrementMouseDown = React.useCallback(function (event) {
        updateValue(true /* shouldSpin */, internalState.initialStepDelay, onDecrement, event);
    }, []);
    return (React.createElement("div", { className: classNames.root, ref: ref },
        labelPosition !== Position.bottom && (iconProps || label) && (React.createElement("div", { className: classNames.labelWrapper },
            iconProps && React.createElement(Icon, __assign({}, iconProps, { className: classNames.icon, "aria-hidden": "true" })),
            label && (React.createElement(Label, { id: internalState.labelId, htmlFor: internalState.inputId, className: classNames.label, disabled: disabled }, label)))),
        React.createElement("div", __assign({}, nativeProps, { className: classNames.spinButtonWrapper, "aria-label": ariaLabel && ariaLabel, "aria-posinset": ariaPositionInSet, "aria-setsize": ariaSetSize, "data-ktp-target": true }),
            React.createElement("input", __assign({ value: value, id: internalState.inputId, onChange: onChange, onInput: onInputChange, className: classNames.input, type: "text", autoComplete: "off", role: "spinbutton", "aria-labelledby": label && internalState.labelId, "aria-valuenow": typeof ariaValueNow === 'number'
                    ? ariaValueNow
                    : value && !isNaN(Number(value)) // Number('') is 0 which may not be desirable
                        ? Number(value)
                        : undefined, "aria-valuetext": ariaValueText ? ariaValueText : !value || isNaN(Number(value)) ? value : undefined, "aria-valuemin": min, "aria-valuemax": max, "aria-describedby": ariaDescribedBy, onBlur: onBlur, ref: input, onFocus: onFocus, onKeyDown: handleKeyDown, onKeyUp: handleKeyUp, disabled: disabled, "aria-disabled": disabled, "data-lpignore": true, "data-ktp-execute-target": true }, inputProps)),
            React.createElement("span", { className: classNames.arrowButtonsContainer },
                React.createElement(IconButton, __assign({ styles: getArrowButtonStyles(theme, true, customUpArrowButtonStyles), className: 'ms-UpButton', checked: keyboardSpinDirection === KeyboardSpinDirection.up, disabled: disabled, iconProps: incrementButtonIcon, onMouseDown: onIncrementMouseDown, onMouseLeave: stop, onMouseUp: stop, tabIndex: -1, ariaLabel: incrementButtonAriaLabel, "data-is-focusable": false }, iconButtonProps)),
                React.createElement(IconButton, __assign({ styles: getArrowButtonStyles(theme, false, customDownArrowButtonStyles), className: 'ms-DownButton', checked: keyboardSpinDirection === KeyboardSpinDirection.down, disabled: disabled, iconProps: decrementButtonIcon, onMouseDown: onDecrementMouseDown, onMouseLeave: stop, onMouseUp: stop, tabIndex: -1, ariaLabel: decrementButtonAriaLabel, "data-is-focusable": false }, iconButtonProps)))),
        labelPosition === Position.bottom && (iconProps || label) && (React.createElement("div", { className: classNames.labelWrapper },
            iconProps && React.createElement(Icon, { iconName: iconProps.iconName, className: classNames.icon, "aria-hidden": "true" }),
            label && (React.createElement(Label, { id: internalState.labelId, htmlFor: internalState.inputId, className: classNames.label, disabled: disabled }, label))))));
});
SpinButtonBase.displayName = COMPONENT_NAME;
//# sourceMappingURL=SpinButton.base.js.map